@@ -7,7 +7,7 @@ use GraphQL::Debug qw(_debug);
 use Lingua::EN::Inflect::Number qw(to_S);
 use Carp qw(confess);
 
-our $VERSION = "0.09";
+our $VERSION = "0.10";
 use constant DEBUG => $ENV{GRAPHQL_DEBUG};
 
 my %GRAPHQL_TYPE2SQLS = (
@@ -239,10 +239,12 @@ sub to_graphql {
   my @ast;
   my (
     %name2type, %name2column21, %name2pk21, %name2fk21, %name2rel21,
-    %name2column2rawtype, %seentype,
+    %name2column2rawtype, %seentype, %name2isview,
   );
   for my $source (map $dbic_schema->source($_), $dbic_schema->sources) {
     my $name = _dbicsource2pretty($source);
+    DEBUG and _debug("schema_dbic2graphql($name)", $source);
+    $name2isview{$name} = 1 if $source->can('view_definition');
     my %fields;
     my $columns_info = $source->columns_info;
     $name2pk21{$name} = +{ map { ($_ => 1) } $source->primary_columns };
@@ -275,6 +277,7 @@ sub to_graphql {
       my $type = _dbicsource2pretty($info->{source});
       $rel =~ s/_id$//; # dumb heuristic
       delete $name2column21{$name}->{$rel}; # so it's not a "column" now
+      delete $name2pk21{$name}{$rel}; # it's not a PK either
       # if it WAS a column, capture its non-null-ness
       my $non_null = ref(($fields{$rel} || {})->{type}) eq 'ARRAY';
       $type = _apply_modifier('non_null', $type) if $non_null;
@@ -294,7 +297,7 @@ sub to_graphql {
   push @ast, map _type2createinput(
     $_, $name2type{$_}->{fields}, \%name2pk21, $name2fk21{$_},
     $name2column21{$_}, \%name2type,
-  ), keys %name2type;
+  ), grep !$name2isview{$_}, keys %name2type;
   push @ast, map _type2searchinput(
     $_, $name2column2rawtype{$_}, \%name2pk21,
     $name2column21{$_}, \%name2type,
@@ -302,7 +305,7 @@ sub to_graphql {
   push @ast, map _type2mutateinput(
     $_, $name2column2rawtype{$_}, $name2type{$_}->{fields}, \%name2pk21,
     $name2column21{$_},
-  ), keys %name2type;
+  ), grep !$name2isview{$_}, keys %name2type;
   push @ast, {
     kind => 'type',
     name => 'Query',
@@ -343,7 +346,7 @@ sub to_graphql {
         };
         (
           # the PKs query
-          $pksearch_name => {
+          keys %{ $name2pk21{$name} } ? ($pksearch_name => {
             type => _apply_modifier('list', $name),
             args => {
               map {
@@ -354,7 +357,7 @@ sub to_graphql {
                 }
               } keys %{ $name2pk21{$name} }
             },
-          },
+          }) : (),
           $input_search_name => {
             description => 'input to search',
             type => _apply_modifier('list', $name),
@@ -469,7 +472,7 @@ sub to_graphql {
             },
           },
         )
-      } keys %name2type
+      } grep !$name2isview{$_}, keys %name2type
     },
   };
   +{
